@@ -11,12 +11,12 @@
 # This file is part of nala
 # nala is based upon apt-metalink https://github.com/tatsuhiro-t/apt-metalink
 #
-# nala is program is free software: you can redistribute it and/or modify
+# nala is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
-# nala is program is distributed in the hope that it will be useful,
+# nala is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
@@ -26,8 +26,10 @@
 """Functions for the Nala Install command."""
 from __future__ import annotations
 
+import fnmatch
 import sys
 from pathlib import Path
+from shutil import which
 from typing import Iterable, cast
 
 import apt_pkg
@@ -82,7 +84,12 @@ from nala.utils import (
 
 def auto_remover(cache: Cache, nala_pkgs: PackageHandler, config: bool = False) -> None:
 	"""Handle auto removal of packages."""
-	if not arguments.auto_remove or arguments.command in ("autoremove", "autopurge"):
+	dprint("Starting Auto Remover")
+	if not arguments.auto_remove and arguments.command not in (
+		"autoremove",
+		"autopurge",
+	):
+		dprint("Packages will not be autoremoved")
 		nala_pkgs.not_needed = [
 			NalaPackage(pkg.name, pkg.installed.version, pkg.installed.installed_size)
 			for pkg in cache
@@ -90,6 +97,7 @@ def auto_remover(cache: Cache, nala_pkgs: PackageHandler, config: bool = False)
 		]
 		return
 
+	dprint("Auto-Removing")
 	with cache.actiongroup():  # type: ignore[attr-defined]
 		# Recurse 10 levels if we're installing .debs to make sure that all depends are safe
 		deps = recurse_deps(nala_pkgs.local_debs, levels=10, installed=False)
@@ -222,10 +230,59 @@ def fix_excluded(protected: list[Package], is_upgrade: Iterable[Package]) -> lis
 	return sorted(new_pkg | old_pkg)
 
 
+def hook_exists(key: str, pkg_names: set[str]) -> bool:
+	"""Return True if the hook file exists on the system."""
+	# cast because mypy thinks it's returning a list
+	return cast(
+		bool,
+		(
+			key == "hook"
+			or key in pkg_names
+			or ("*" in key and fnmatch.filter(pkg_names, key))
+		),
+	)
+
+
+def run_hooks(pkg_names: set[str], hook_type: str, install: InstallProgress) -> None:
+	"""Run the install hooks."""
+	dprint(f"{hook_type} {(hooks := arguments.config.get_hook(hook_type).items())}\n")
+	for key, hook in hooks:
+		dprint(f"Key: {key}, Hook: {hook}\n")
+		if hook_exists(key, pkg_names):
+			install.run_install(hook.split(), hook=True)
+
+
+def check_hooks(pkg_names: set[str]) -> None:
+	"""Check that the hook paths exist before trying to run anything."""
+	hooks: dict[str, list[str]] = {
+		"PreInstall": [],
+		"PostInstall": [],
+	}
+	for hook_type, hook_list in hooks.items():
+		for key, hook in arguments.config.get_hook(hook_type).items():
+			if hook_exists(key, pkg_names) and not which(cmd := hook.split()[0]):
+				hook_list.append(color(cmd, "YELLOW"))
+
+	if not hooks["PreInstall"] + hooks["PostInstall"]:
+		return
+
+	for hook_type, hook_list in hooks.items():
+		if hook_list:
+			eprint(
+				_("{error} The following {hook_type} commands cannot be found.").format(
+					error=ERROR_PREFIX, hook_type=hook_type
+				)
+			)
+			eprint(f"  {', '.join(hook_list)}")
+	sys.exit(1)
+
+
 def commit_pkgs(cache: Cache, nala_pkgs: PackageHandler) -> None:
 	"""Commit the package changes to the cache."""
 	dprint("Commit Pkgs")
 	task = dpkg_progress.add_task("", total=nala_pkgs.dpkg_progress_total())
+	check_hooks(pkg_names := {pkg.name for pkg in nala_pkgs.all_pkgs()})
+
 	with DpkgLive(install=True) as live:
 		with open(DPKG_LOG, "w", encoding="utf-8") as dpkg_log:
 			with open(NALA_TERM_LOG, "a", encoding="utf-8") as term_log:
@@ -240,9 +297,11 @@ def commit_pkgs(cache: Cache, nala_pkgs: PackageHandler) -> None:
 				)
 				install = InstallProgress(dpkg_log, term_log, live, task, config_purge)
 				update = UpdateProgress(live)
+				run_hooks(pkg_names, "PreInstall", install)
 				cache.commit_pkgs(install, update)
 				if nala_pkgs.local_debs:
 					cache.commit_pkgs(install, update, nala_pkgs.local_debs)
+				run_hooks(pkg_names, "PostInstall", install)
 				term_log.write(
 					_("Log Ended: [{date}]").format(date=get_date()) + "\n\n"
 				)
@@ -763,7 +822,7 @@ def setup_cache() -> Cache:
 				with DpkgLive(install=False) as live:
 					Cache().update(UpdateProgress(live))
 	except (LockFailedException, FetchFailedException, apt_pkg.Error) as err:
-		apt_error(err)
+		apt_error(err, arguments.command == "update")
 	except KeyboardInterrupt:
 		eprint(_("Exiting due to SIGINT"))
 		sys.exit(ExitCode.SIGINT)
